@@ -48,9 +48,7 @@
 #include <audacious/plugin.h>
 #include <audacious/i18n.h>
 #include <audacious/output.h>
-#include <audacious/playlist.h>
 #include <audacious/ui_plugin_menu.h>
-#include <audacious/util.h>
 
 #include "cdaudio-ng.h"
 #include "configure.h"
@@ -67,8 +65,8 @@ static dae_params_t		*pdae_params = NULL;
 static InputPlayback    	*pglobalinputplayback = NULL;
 static GtkWidget		*main_menu_item, *playlist_menu_item;
 static GThread			*scan_cd_thread = NULL;
-static int			first_trackno_to_add_after_scan = -1;
-static int			last_trackno_to_add_after_scan = -1;
+static gint			first_trackno_to_add_after_scan = -1;
+static gint			last_trackno_to_add_after_scan = -1;
 
 static void			cdaudio_init(void);
 static void			cdaudio_about(void);
@@ -86,6 +84,7 @@ static void			cdaudio_get_song_info(gchar *filename, gchar **title, gint *length
 static Tuple            	*cdaudio_get_song_tuple(gchar *filename);
 
 static void			menu_click(void);
+static void			rescan_menu_click(void);
 static Tuple            	*create_tuple_from_trackinfo_and_filename(gchar *filename);
 static void			dae_play_loop(dae_params_t *pdae_params);
 static void			*scan_cd(void *nothing);
@@ -144,7 +143,7 @@ static void debug(const char *fmt, ...)
 
 static void cdaudio_init()
 {
-	ConfigDb *db;
+	mcs_handle_t *db;
 	gchar *menu_item_text;
 
 	debug("cdaudio_init()\n");
@@ -155,6 +154,7 @@ static void cdaudio_init()
 	cdng_cfg.debug = FALSE;
 	cdng_cfg.device = g_strdup("");
 	cdng_cfg.cddb_server = g_strdup(CDDA_DEFAULT_CDDB_SERVER);
+	cdng_cfg.cddb_path = g_strdup("");
 	cdng_cfg.cddb_port = CDDA_DEFAULT_CDDB_PORT;
 	cdng_cfg.cddb_http = FALSE;
 	cdng_cfg.limitspeed = CDDA_DEFAULT_LIMIT_SPEED;
@@ -185,6 +185,7 @@ static void cdaudio_init()
 	aud_cfg_db_get_bool(db, "CDDA", "debug", &cdng_cfg.debug);
 	aud_cfg_db_get_string(db, "CDDA", "device", &cdng_cfg.device);
 	aud_cfg_db_get_string(db, "CDDA", "cddbserver", &cdng_cfg.cddb_server);
+	aud_cfg_db_get_string(db, "CDDA", "cddbpath", &cdng_cfg.cddb_path);
 	aud_cfg_db_get_int(db, "CDDA", "cddbport", &cdng_cfg.cddb_port);
 	aud_cfg_db_get_bool(db, "CDDA", "cddbhttp", &cdng_cfg.cddb_http);
 	aud_cfg_db_get_int(db, "CDDA", "limitspeed", &cdng_cfg.limitspeed);
@@ -196,11 +197,22 @@ static void cdaudio_init()
 
 	aud_cfg_db_close(db);
 
-	debug("use_dae = %d, limitspeed = %d, use_cdtext = %d, use_cddb = %d, cddbserver = \"%s\", cddbport = %d, cddbhttp = %d, device = \"%s\", debug = %d\n",
+	debug("use_dae = %d, limitspeed = %d, use_cdtext = %d, use_cddb = %d, cddbserver = \"%s\", cddbpath = \"%s\", cddbport = %d, cddbhttp = %d, device = \"%s\", debug = %d\n",
 		cdng_cfg.use_dae, cdng_cfg.limitspeed, cdng_cfg.use_cdtext, cdng_cfg.use_cddb,
-		cdng_cfg.cddb_server, cdng_cfg.cddb_port, cdng_cfg.cddb_http, cdng_cfg.device, cdng_cfg.debug);
+          cdng_cfg.cddb_server, cdng_cfg.cddb_path, cdng_cfg.cddb_port, cdng_cfg.cddb_http, cdng_cfg.device, cdng_cfg.debug);
 
-	configure_create_gui();
+	menu_item_text = _("Rescan CD");
+	main_menu_item = gtk_image_menu_item_new_with_label(menu_item_text);
+	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(main_menu_item), gtk_image_new_from_stock(GTK_STOCK_REFRESH, GTK_ICON_SIZE_MENU));
+	gtk_widget_show(main_menu_item);
+	audacious_menu_plugin_item_add(AUDACIOUS_MENU_MAIN, main_menu_item);
+	g_signal_connect(G_OBJECT(main_menu_item), "activate", G_CALLBACK(rescan_menu_click), NULL);
+
+	playlist_menu_item = gtk_image_menu_item_new_with_label(menu_item_text);
+	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(playlist_menu_item), gtk_image_new_from_stock(GTK_STOCK_CDROM, GTK_ICON_SIZE_MENU));
+	gtk_widget_show(playlist_menu_item);
+	audacious_menu_plugin_item_add(AUDACIOUS_MENU_PLAYLIST_RCLICK, playlist_menu_item);
+	g_signal_connect(G_OBJECT(playlist_menu_item), "activate", G_CALLBACK(menu_click), NULL);
 
 	menu_item_text = _("Add CD");
 	main_menu_item = gtk_image_menu_item_new_with_label(menu_item_text);
@@ -244,9 +256,9 @@ static void cdaudio_configure()
 
 	configure_show_gui();
 
-	debug("use_dae = %d, limitspeed = %d, use_cdtext = %d, use_cddb = %d, cddbserver = \"%s\", cddbport = %d, cddbhttp = %d, device = \"%s\", debug = %d\n",
+	debug("use_dae = %d, limitspeed = %d, use_cdtext = %d, use_cddb = %d, cddbserver = \"%s\", cddbpath = \"%s\", cddbport = %d, cddbhttp = %d, device = \"%s\", debug = %d\n",
 		cdng_cfg.use_dae, cdng_cfg.limitspeed, cdng_cfg.use_cdtext, cdng_cfg.use_cddb,
-		cdng_cfg.cddb_server, cdng_cfg.cddb_port, cdng_cfg.cddb_http, cdng_cfg.device, cdng_cfg.debug);
+		cdng_cfg.cddb_server, cdng_cfg.cddb_path, cdng_cfg.cddb_port, cdng_cfg.cddb_http, cdng_cfg.device, cdng_cfg.debug);
 }
 
 static gint cdaudio_is_our_file(gchar *filename)
@@ -572,12 +584,13 @@ static void cdaudio_cleanup(void)
 
 	// todo: destroy the gui
 
-	ConfigDb *db = aud_cfg_db_open();
+	mcs_handle_t *db = aud_cfg_db_open();
 	aud_cfg_db_set_bool(db, "CDDA", "use_dae", cdng_cfg.use_dae);
 	aud_cfg_db_set_int(db, "CDDA", "limitspeed", cdng_cfg.limitspeed);
 	aud_cfg_db_set_bool(db, "CDDA", "use_cdtext", cdng_cfg.use_cdtext);
 	aud_cfg_db_set_bool(db, "CDDA", "use_cddb", cdng_cfg.use_cddb);
 	aud_cfg_db_set_string(db, "CDDA", "cddbserver", cdng_cfg.cddb_server);
+	aud_cfg_db_set_string(db, "CDDA", "cddbpath", cdng_cfg.cddb_path);
 	aud_cfg_db_set_int(db, "CDDA", "cddbport", cdng_cfg.cddb_port);
 	aud_cfg_db_set_bool(db, "CDDA", "cddbhttp", cdng_cfg.cddb_http);
 	aud_cfg_db_set_string(db, "CDDA", "device", cdng_cfg.device);
@@ -614,7 +627,7 @@ static Tuple *cdaudio_get_song_tuple(gchar *filename)
 /*
  * auxiliar functions
  */
-static void menu_click()
+static void menu_click(void)
 {
 	debug("plugin services menu option selected\n");
 
@@ -628,12 +641,19 @@ static void menu_click()
 		}
 	}
 	else {
-		int trackno;
+		gint trackno;
 		for (trackno = firsttrackno; trackno <= lasttrackno; trackno++)
 			append_track_to_playlist(trackno);
 	}
 }
 
+static void rescan_menu_click(void)
+{
+	debug("plugin services rescan option selected\n");
+	
+	scan_cd_threaded(0, 0);
+}
+
 static Tuple *create_tuple_from_trackinfo_and_filename(gchar *filename)
 {
 	Tuple *tuple = aud_tuple_new_from_filename(filename);
@@ -964,6 +984,7 @@ static void *scan_cd(void *nothing)
 						cddb_http_enable(pcddb_conn);
 						cddb_set_server_name(pcddb_conn, cdng_cfg.cddb_server);
 						cddb_set_server_port(pcddb_conn, cdng_cfg.cddb_port);
+                        cddb_set_http_path_query(pcddb_conn, cdng_cfg.cddb_path);
 					}
 					else {
 						cddb_set_server_name(pcddb_conn, cdng_cfg.cddb_server);
@@ -982,7 +1003,7 @@ static void *scan_cd(void *nothing)
 				}
 
 				cddb_disc_calc_discid(pcddb_disc);
-				unsigned int discid = cddb_disc_get_discid(pcddb_disc);
+				guint discid = cddb_disc_get_discid(pcddb_disc);
 				debug("CDDB disc id = %x\n", discid);
 
 				gint matches;
@@ -1018,7 +1039,7 @@ static void *scan_cd(void *nothing)
 								cddb_disc_get_title(pcddb_disc),
 								cddb_disc_get_genre(pcddb_disc));
 
-							int trackno;
+							gint trackno;
 							for (trackno = firsttrackno; trackno <= lasttrackno; trackno++) {
 								cddb_track_t *pcddb_track = cddb_disc_get_track(pcddb_disc, trackno - 1);
 								cdaudio_set_strinfo(&trackinfo[trackno],
