@@ -25,6 +25,7 @@
 #include "ed_bindings_store.h"
 #include "ed_common.h"
 #include <stdlib.h>
+#include <string.h>
 #include <linux/input.h>
 
 #include <audacious/i18n.h>
@@ -536,7 +537,6 @@ static void
 cfg_config_cb_commit ( gpointer cfg_device_lv )
 {
   GList *config_device_list = NULL;
-  GList *list_iter;
   GtkTreeModel *model;
 
   model = gtk_tree_view_get_model( GTK_TREE_VIEW(cfg_device_lv) );
@@ -757,7 +757,6 @@ cfg_bindbox_assign_binding_input_func ( GIOChannel * iochan ,
     case G_IO_IN:
     {
       gsize rb = 0;
-      GError *gerr = NULL;
       struct input_event inputev;
       g_io_channel_read_chars( iochan , (gchar*)&inputev ,
                                sizeof(struct input_event) , &rb , NULL );
@@ -792,6 +791,9 @@ cfg_bindbox_assign_binding_input_func ( GIOChannel * iochan ,
         }
       }
     }
+
+  default:
+    ;
   }
   return TRUE;
 }
@@ -802,7 +804,7 @@ cfg_bindbox_assign_binding_checkdups( GtkWidget * table , ed_inputevent_t * inpu
 {
   /* check if inputev is already assigned in table */
   GList *children = GTK_TABLE(table)->children;
-  for ( children ; children != NULL ; children = g_list_next(children) )
+  for ( ; children != NULL ; children = g_list_next(children) )
   {
     GtkTableChild *child = children->data;
 
@@ -1070,7 +1072,7 @@ cfg_bindbox_populate( GtkWidget * bind_table , GtkWidget * bindings_win , gpoint
     for ( i = 0 ; i < bindings_num ; i++ )
     {
       GList *children = GTK_TABLE(bind_table)->children;
-      for ( children ; children != NULL ; children = g_list_next(children) )
+      for ( ; children != NULL ; children = g_list_next(children) )
       {
         GtkTableChild *child = children->data;
         if ( ( (child->top_attach + 1) == GTK_TABLE(bind_table)->nrows ) &&
@@ -1082,7 +1084,6 @@ cfg_bindbox_populate( GtkWidget * bind_table , GtkWidget * bindings_win , gpoint
           GtkWidget *label = g_object_get_data(G_OBJECT(child->widget),"label");
           GtkWidget *delbt = g_object_get_data(G_OBJECT(child->widget),"delbt");
           GtkTreeModel *combomodel;
-          GtkTreeIter comboiter;
           combosas_helper_t *combosas_helper;
           gchar *input_str;
 
@@ -1235,7 +1236,7 @@ cfg_bindings_cb_commit ( gpointer bindings_win )
       array_actioncode = calloc( table->nrows - 1 , sizeof(gint) );
       array_inputevent = calloc( table->nrows - 1 , sizeof(ed_inputevent_t*) );
 
-      for ( children ; children != NULL ; children = g_list_next( children ) )
+      for ( ; children != NULL ; children = g_list_next( children ) )
       {
         /* pick information from relevant table cells and put them in arrays */
         GtkTableChild *child = children->data;
@@ -1445,6 +1446,7 @@ ed_ui_about_show( void )
   GtkWidget *bbar_bbox, *bbar_bt_ok;
   GtkTextBuffer *info_tb;
   GdkGeometry abount_win_hints;
+  gchar *info_tb_content = NULL;
 
   if ( about_win != NULL )
   {
@@ -1485,12 +1487,13 @@ ed_ui_about_show( void )
   gtk_text_view_set_justification( GTK_TEXT_VIEW(info_tv) , GTK_JUSTIFY_LEFT );
   gtk_text_view_set_left_margin( GTK_TEXT_VIEW(info_tv) , 10 );
 
-  gtk_text_buffer_set_text( info_tb ,
-                            _("\nEvDev-Plug " ED_VERSION_PLUGIN
-                              "\nplayer remote control via event devices\n"
-                              "http://www.develia.org/projects.php?p=evdevplug\n\n"
-                              "written by Giacomo Lozito\n"
-                              "< james@develia.org >\n\n") , -1 );
+  info_tb_content = g_strjoin( NULL , "\nEvDev-Plug " , ED_VERSION_PLUGIN ,
+                               _("\nplayer remote control via event devices\n"
+                                 "http://www.develia.org/projects.php?p=audacious#evdevplug\n\n"
+                                 "written by Giacomo Lozito\n") ,
+                              "< james@develia.org >\n\n" , NULL );
+  gtk_text_buffer_set_text( info_tb , info_tb_content , -1 );
+  g_free( info_tb_content );
 
   info_tv_sw = gtk_scrolled_window_new( NULL , NULL );
   gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW(info_tv_sw) ,
