@@ -19,146 +19,54 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+/* #define AUD_DEBUG 1 */
+
 #include <math.h>
 #include <assert.h>
-#include <pthread.h>
-#include <signal.h>
 
 #include <audacious/plugin.h>
-#include <audacious/output.h>
-#include <audacious/util.h>
-#include <sys/time.h>
 #include "plugin.h"
 #include "input.h"
 
-#define BUFFER_SIZE 16*1024
+#define BUFFER_SIZE (16*1024)
 #define N_AVERAGE_FRAMES 10
 
-extern int triangular_dither_noise(int nbits);
-
-/**
- * Scale PCM data
- */
-static inline signed int
-scale(mad_fixed_t sample, struct mad_info_t *file_info)
-{
-    /* replayGain by SamKR */
-    gdouble scale = -1;
-    if (audmad_config.replaygain.enable) {
-        if (file_info->has_replaygain) {
-            scale = file_info->replaygain_track_scale;
-            if (file_info->replaygain_album_scale != -1
-                && (scale == -1 || !audmad_config.replaygain.track_mode))
-            {
-                scale = file_info->replaygain_album_scale;
-            }
-        }
-        if (scale == -1)
-            scale = audmad_config.replaygain.default_scale;
-    }
-    if (scale == -1)
-        scale = 1.0;
-    if (audmad_config.pregain_scale != 1)
-        scale = scale * audmad_config.pregain_scale;
-
-    /* hard-limit (clipping-prevention) */
-    if (audmad_config.hard_limit) {
-        /* convert to double before computation, to avoid mad_fixed_t wrapping */
-        double x = mad_f_todouble(sample) * scale;
-        static const double k = 0.5;    // -6dBFS
-        if (x > k) {
-            x = tanh((x - k) / (1 - k)) * (1 - k) + k;
-        }
-        else if (x < -k) {
-            x = tanh((x + k) / (1 - k)) * (1 - k) - k;
-        }
-        sample = x * (MAD_F_ONE);
-    }
-    else
-        sample *= scale;
-
-    int n_bits_to_loose = MAD_F_FRACBITS + 1 - 16;
-
-    /* round */
-    /* add half of the bits_to_loose range to round */
-    sample += (1L << (n_bits_to_loose - 1));
-
-#ifdef DEBUG_DITHER
-    mad_fixed_t no_dither = sample;
-#endif
-
-    /* dither one bit of actual output */
-    if (audmad_config.dither) {
-        int dither = triangular_dither_noise(n_bits_to_loose + 1);
-        sample += dither;
-    }
-
-    /* clip */
-    /* make sure we are between -1 and 1 */
-    if (sample >= MAD_F_ONE) {
-        sample = MAD_F_ONE - 1;
-    }
-    else if (sample < -MAD_F_ONE) {
-        sample = -MAD_F_ONE;
-    }
-
-    /* quantize */
-    /*
-     * Turn our mad_fixed_t into an integer.
-     * Shift all but 16-bits of the fractional part
-     * off the right hand side and shift an extra place
-     * to get the sign bit.
-     */
-    sample >>= n_bits_to_loose;
-#ifdef DEBUG_DITHER
-    static int n_zeros = 0;
-    no_dither >>= n_bits_to_loose;
-    if (no_dither - sample == 0)
-        n_zeros++;
-    else {
-        g_message("dither: %d %d", n_zeros, no_dither - sample);
-        n_zeros = 0;
-    }
-#endif
-    return sample;
-}
-
 void
 write_output(struct mad_info_t *info, struct mad_pcm *pcm,
              struct mad_header *header)
 {
     unsigned int nsamples;
     mad_fixed_t const *left_ch, *right_ch;
-    char *output;
-    int olen = 0;
+    mad_fixed_t *output;
+    int outlen = 0;
+    int outbyte = 0;
     int pos = 0;
 
     nsamples = pcm->length;
     left_ch = pcm->samples[0];
     right_ch = pcm->samples[1];
-    olen = nsamples * MAD_NCHANNELS(header) * 2;
-    output = (char *) g_malloc(olen * sizeof(char));
+    outlen = nsamples * MAD_NCHANNELS(header);
+    outbyte = outlen * sizeof(mad_fixed_t);
+
+    output = (mad_fixed_t *) g_malloc(outbyte);
 
     while (nsamples--) {
-        signed int sample;
-        /* output sample(s) in 16-bit signed little-endian PCM */
-        sample = scale(*left_ch++, info);
-        output[pos++] = (sample >> 0) & 0xff;
-        output[pos++] = (sample >> 8) & 0xff;
+        output[pos++] = *left_ch++;
 
         if (MAD_NCHANNELS(header) == 2) {
-            sample = scale(*right_ch++, info);
-            output[pos++] = (sample >> 0) & 0xff;
-            output[pos++] = (sample >> 8) & 0xff;
+            output[pos++] = *right_ch++;
         }
     }
-    assert(pos == olen);
-    if (!info->playback->playing)
-        return;
-    produce_audio(info->playback->output->written_time(),
-                  FMT_S16_LE, MAD_NCHANNELS(header), olen, output, &(info->playback->playing));
-    if (!info->playback->playing)
+
+    assert(pos == outlen);
+    if (!info->playback->playing) {
+        g_free(output);
         return;
+    }
+
+    info->playback->pass_audio(info->playback,
+                               info->fmt, MAD_NCHANNELS(header), outbyte, output,
+                               &(info->playback->playing));
     g_free(output);
 }
 
@@ -166,7 +74,8 @@ write_output(struct mad_info_t *info, struct mad_pcm *pcm,
  * Decode all headers in the file and fill in stats
  * @return FALSE if scan failed.
  */
-gboolean scan_file(struct mad_info_t * info, gboolean fast)
+gboolean
+scan_file(struct mad_info_t * info, gboolean fast)
 {
     struct mad_stream stream;
     struct mad_header header;
@@ -192,37 +101,28 @@ gboolean scan_file(struct mad_info_t * info, gboolean fast)
     info->duration = mad_timer_zero; // should be cleared before loop, if we use it as break condition.
 
     if(info->fileinfo_request == TRUE) {
-        info->tuple->length = -1;
+        aud_tuple_associate_int(info->tuple, FIELD_LENGTH, NULL, -1);
         info->fileinfo_request = FALSE;
     }
 
-#ifdef DEBUG
-    g_message("f: scan_file");
-    g_message("scan_file frames = %d", info->frames);
-#endif                          /* DEBUG */
+    AUDDBG("f: scan_file\n");
+    AUDDBG("scan_file frames = %d\n", info->frames);
 
     while (1) {
         remainder = stream.bufend - stream.next_frame;
-	
-        /*
-           if (remainder >= BUFFER_SIZE)
-           {
-           printf("oh dear.. remainder = %d\n", remainder);
-           }
-         */
 
-        memcpy(buffer, stream.this_frame, remainder);
+        if(buffer != stream.this_frame && remainder)
+            memmove(buffer, stream.this_frame, remainder);
+
         len = input_get_data(info, buffer + remainder,
                              BUFFER_SIZE - remainder);
 
         if (len <= 0) {
-#ifdef DEBUG
-            g_message("scan_file: len <= 0 len = %d", len);
-#endif
+            AUDDBG("scan_file: len <= 0 len = %d\n", len);
             break;
         }
 
-        mad_stream_buffer(&stream, buffer, len + remainder);
+        mad_stream_buffer(&stream, buffer, remainder + len);
 
         while (!fast || (fast && info->frames < N_AVERAGE_FRAMES)) {
             if (mad_header_decode(&header, &stream) == -1) {
@@ -230,12 +130,10 @@ gboolean scan_file(struct mad_info_t * info, gboolean fast)
                     break;
                 }
                 if (!MAD_RECOVERABLE(stream.error)) {
-#ifdef DEBUG
-                    g_message("(fatal) error decoding header %d: %s",
+                    AUDDBG("(fatal) error decoding header %d: %s\n",
                               info->frames, mad_stream_errorstr(&stream));
-                    g_message("remainder = %d", remainder);
-                    g_message("len = %d", len);
-#endif                          /* DEBUG */
+                    AUDDBG("remainder = %d\n", remainder);
+                    AUDDBG("len = %d\n", len);
                     break;
                 }
                 if (stream.error == MAD_ERROR_LOSTSYNC) {
@@ -244,37 +142,35 @@ gboolean scan_file(struct mad_info_t * info, gboolean fast)
                                             stream.bufend -
                                             stream.this_frame);
                     if (tagsize > 0) {
-#ifdef DEBUG
-                        g_message("skipping id3_tag: %d", tagsize);
-#endif                          /* DEBUG */
+                        AUDDBG("skipping id3_tag: %d\n", tagsize);
                         mad_stream_skip(&stream, tagsize);
                         continue;
                     }
                 }
-#ifdef DEBUG
-                g_message("(recovered) error decoding header %d: %s",
+
+                AUDDBG("(recovered) error decoding header %d: %s\n",
                           info->frames, mad_stream_errorstr(&stream));
-                g_message("remainder = %d", remainder);
-                g_message("len = %d", len);
-#endif                          /* DEBUG */
+                AUDDBG("remainder = %d\n", remainder);
+                AUDDBG("len = %d\n", len);
+
                 continue;
             }
             info->frames++;
 
-#ifdef DEBUG
 #ifdef DEBUG_INTENSIVELY
-            g_message("header bitrate = %ld", header.bitrate);
-            g_message("duration = %ul",
+            AUDDBG("header bitrate = %ld\n", header.bitrate);
+            AUDDBG("duration = %ul\n",
                       mad_timer_count(header.duration,
                                       MAD_UNITS_MILLISECONDS));
-            g_message("size = %d", stream.next_frame - stream.this_frame);
+            AUDDBG("size = %d\n", stream.next_frame - stream.this_frame);
 #endif
-#endif
-            if(info->tuple->length == -1)
+            if(aud_tuple_get_int(info->tuple, FIELD_LENGTH, NULL) == -1)
                 mad_timer_add(&info->duration, header.duration);
             else {
-                info->duration.seconds = info->tuple->length / 1000;
-                info->duration.fraction = info->tuple->length % 1000;
+                gint length = aud_tuple_get_int(info->tuple, FIELD_LENGTH, NULL);
+
+                info->duration.seconds = length / 1000;
+                info->duration.fraction = length % 1000;
             }
             data_used += stream.next_frame - stream.this_frame;
             if (info->frames == 1) {
@@ -285,43 +181,37 @@ gboolean scan_file(struct mad_info_t * info, gboolean fast)
                 info->mpeg_layer = header.layer;
                 info->mode = header.mode;
 
-                if (audmad_config.use_xing) {
+                if (audmad_config->use_xing) {
                     frame.header = header;
                     if (mad_frame_decode(&frame, &stream) == -1) {
-#ifdef DEBUG
-                        g_message("xing frame decode failed");
-#endif
+                        AUDDBG("xing frame decode failed\n");
                         goto no_xing;
                     }
                     if (xing_parse(&info->xing, stream.anc_ptr, stream.anc_bitlen) == 0) {
-#ifdef DEBUG
-                        g_message("xing header found ");
-#endif                          /* DEBUG */
+                        AUDDBG("xing header found\n");
                         has_xing = TRUE;
                         info->vbr = TRUE;   /* otherwise xing header would have been 'Info' */
 
-#ifdef DEBUG
-                        g_message("xing: bytes = %ld frames = %ld", info->xing.bytes, info->xing.frames);
-#endif
+                        AUDDBG("xing: bytes = %ld frames = %ld\n", info->xing.bytes, info->xing.frames);
+
                         /* we have enough info to calculate bitrate and duration */
                         if(info->xing.bytes && info->xing.frames) {
                             xing_bitrate = 8 * (double)info->xing.bytes * 38 / (double)info->xing.frames; //38fps in MPEG1.
-#ifdef DEBUG
+#ifdef AUD_DEBUG
                             {
                                 gint tmp = (gint)(info->xing.bytes * 8 / xing_bitrate);
-                                g_message("xing: bitrate = %4.1f kbps", xing_bitrate / 1000);
-                                g_message("xing: duration = %d:%02d", tmp / 60, tmp % 60);
+                                AUDDBG("xing: bitrate = %4.1f kbps\n", xing_bitrate / 1000);
+                                AUDDBG("xing: duration = %d:%02d\n", tmp / 60, tmp % 60);
                             }
 #endif
                         }
                         continue;
                     }
-#ifdef DEBUG
+#ifdef AUD_DEBUG
                     else {
-                        g_message("no usable xing header");
+                        AUDDBG("no usable xing header\n");
                         continue;
                     }
-                    
 #endif
                 } /* xing */
 
@@ -345,17 +235,17 @@ gboolean scan_file(struct mad_info_t * info, gboolean fast)
         no_xing:
             if (fast && info->frames >= N_AVERAGE_FRAMES) {
                 float frame_size = ((double) data_used) / N_AVERAGE_FRAMES;
-#ifdef DEBUG
-                g_message("bitrate = %ld samplerate = %d", header.bitrate, header.samplerate);
-                g_message("data_used = %d info->frames = %d info->size = %d tagsize = %d frame_size = %lf",
+
+                AUDDBG("bitrate = %ld samplerate = %d\n", header.bitrate, header.samplerate);
+                AUDDBG("data_used = %d info->frames = %d info->size = %d tagsize = %d frame_size = %lf\n",
                           data_used, info->frames, info->size, tagsize, frame_size);
-#endif
+
                 if(info->size != 0)
                     info->frames = (info->size - tagsize) / frame_size;
-#ifdef DEBUG
-                g_message("info->frames = %d", info->frames);
-#endif
-                if(info->tuple->length == -1) {
+
+                AUDDBG("info->frames = %d\n", info->frames);
+
+                if(aud_tuple_get_int(info->tuple, FIELD_LENGTH, NULL) == -1) {
                     if(xing_bitrate > 0.0) {
                         /* calc duration with xing info */
                         double tmp = 8 * (double)info->xing.bytes * 1000 / xing_bitrate;
@@ -369,18 +259,20 @@ gboolean scan_file(struct mad_info_t * info, gboolean fast)
                     }
                 }
                 else {
-                    info->duration.seconds = info->tuple->length / 1000;
-                    info->duration.fraction = info->tuple->length % 1000;
+                    gint length = aud_tuple_get_int(info->tuple, FIELD_LENGTH, NULL);
+
+                    info->duration.seconds = length / 1000;
+                    info->duration.fraction = length % 1000;
                 }
-#ifdef DEBUG
-                g_message("using fast playtime calculation");
-                g_message("data used = %d [tagsize=%d framesize=%f]",
+#ifdef AUD_DEBUG
+                AUDDBG("using fast playtime calculation\n");
+                AUDDBG("data used = %d [tagsize=%d framesize=%f]\n",
                           data_used, tagsize, frame_size);
-                g_message("frames = %d, frequency = %d, channels = %d",
+                AUDDBG("frames = %d, frequency = %d, channels = %d\n",
                           info->frames, info->freq, info->channels);
                 long millis = mad_timer_count(info->duration,
                                               MAD_UNITS_MILLISECONDS);
-                g_message("duration = %ld:%02ld", millis / 1000 / 60, (millis / 1000) % 60);
+                AUDDBG("duration = %ld:%02ld\n", millis / 1000 / 60, (millis / 1000) % 60);
 #endif                          /* DEBUG */
                 break;
             }
@@ -398,23 +290,25 @@ gboolean scan_file(struct mad_info_t * info, gboolean fast)
     else if (info->vbr && xing_bitrate == 0 && bitrate_frames != 0) {
         info->bitrate = accum_bitrate / bitrate_frames;
     }
+    
+    aud_tuple_associate_int(info->tuple, FIELD_BITRATE, NULL, info->bitrate / 1000);
 
     mad_frame_finish(&frame);
     mad_header_finish(&header);
     mad_stream_finish(&stream);
     xing_finish(&info->xing);
 
-#ifdef DEBUG
-    g_message("scan_file: info->frames = %d", info->frames);
-    g_message("e: scan_file");
-#endif                          /* DEBUG */
+    AUDDBG("scan_file: info->frames = %d\n", info->frames);
+    AUDDBG("e: scan_file\n");
+
     return (info->frames != 0 || info->remote == TRUE);
 }
 
 /* sanity check for audio open parameters */
-static gboolean check_audio_param(struct mad_info_t *info)
+static gboolean
+check_audio_param(struct mad_info_t *info)
 {
-    if(info->fmt < FMT_U8 || info->fmt > FMT_S16_NE)
+    if(info->fmt != FMT_FIXED32 && (info->fmt < FMT_U8 || info->fmt > FMT_S16_NE))
         return FALSE;
     if(info->freq < 0) // not sure about maximum frequency. --yaz
         return FALSE;
@@ -424,7 +318,8 @@ static gboolean check_audio_param(struct mad_info_t *info)
     return TRUE;
 }
 
-gpointer decode_loop(gpointer arg)
+gpointer
+decode_loop(gpointer arg)
 {
     unsigned char buffer[BUFFER_SIZE];
     int len;
@@ -441,9 +336,7 @@ gpointer decode_loop(gpointer arg)
     /* track info is passed in as thread argument */
     struct mad_info_t *info = (struct mad_info_t *) arg;
 
-#ifdef DEBUG
-    g_message("f: decode");
-#endif                          /* DEBUG */
+    AUDDBG("f: decode\n");
 
     /* init mad stuff */
     mad_frame_init(&frame);
@@ -452,15 +345,11 @@ gpointer decode_loop(gpointer arg)
     mad_synth_init(&synth);
 
     if(!info->playback){
-#ifdef DEBUG
-        g_message("decode: playback == NULL");
-#endif
+        AUDDBG("decode: playback == NULL\n");
         return NULL;
     }
 
-#ifdef DEBUG
-    g_message("decode: fmt = %d freq = %d channels = %d", info->fmt, info->freq, info->channels);
-#endif
+    AUDDBG("decode: fmt = %d freq = %d channels = %d\n", info->fmt, info->freq, info->channels);
 
     if(check_audio_param(info) == FALSE)
         return NULL;
@@ -478,40 +367,38 @@ gpointer decode_loop(gpointer arg)
     /* set mainwin title */
     if (info->title)
         g_free(info->title);
-    info->title = xmms_get_titlestring(audmad_config.title_override == TRUE ?
-                                       audmad_config.id3_format : xmms_get_gentitle_format(), info->tuple);
-    
+    info->title = aud_tuple_formatter_make_title_string(info->tuple, audmad_config->title_override == TRUE ?
+                                       audmad_config->id3_format : aud_get_gentitle_format());
+
     tlen = (gint) mad_timer_count(info->duration, MAD_UNITS_MILLISECONDS),
-        mad_plugin->set_info(info->title,
+        info->playback->set_params(info->playback, info->title,
                              (tlen == 0 || info->size <= 0) ? -1 : tlen,
                              info->bitrate, info->freq, info->channels);
-#ifdef DEBUG
-    g_message("decode: tlen = %d", tlen);
-#endif
+
+    AUDDBG("decode: tlen = %d\n", tlen);
 
     /* main loop */
     do {
         if (!info->playback->playing) {
-#ifdef DEBUG
-            g_message("decode: stop signaled");
-#endif                          /* DEBUG */
+            AUDDBG("decode: stop signaled\n");
             break;
         }
+
         if (seek_skip)
             remainder = 0;
         else {
             remainder = stream.bufend - stream.next_frame;
-            memcpy(buffer, stream.this_frame, remainder);
+            if(buffer != stream.this_frame && remainder)
+                memmove(buffer, stream.this_frame, remainder);
         }
+
         len = input_get_data(info, buffer + remainder,
                              BUFFER_SIZE - remainder);
 
         input_process_remote_metadata(info);
 
         if (len <= 0) {
-#ifdef DEBUG
-            g_message("finished decoding");
-#endif                          /* DEBUG */
+            AUDDBG("finished decoding\n");
             break;
         }
         len += remainder;
@@ -525,9 +412,9 @@ gpointer decode_loop(gpointer arg)
         mad_stream_buffer(&stream, buffer, len);
 
         if (seek_skip) {
-#ifdef DEBUG
-            g_message("skipping: %d", seek_skip);
-#endif
+
+            AUDDBG("skipping: %d\n", seek_skip);
+
             int skip = 2;
             do {
                 if (mad_frame_decode(&frame, &stream) == 0) {
@@ -549,25 +436,25 @@ gpointer decode_loop(gpointer arg)
 
         while (info->playback->playing) {
             if (info->seek != -1 && info->size > 0) {
-#ifdef DEBUG
-                g_message("seeking: %ld", info->seek);
-#endif
-                int new_position;
-                int seconds =
-                    mad_timer_count(info->duration, MAD_UNITS_SECONDS);
-                if (info->seek >= seconds)
-                    info->seek = seconds;
 
-                mad_timer_set(&info->pos, info->seek, 0, 0);
+                AUDDBG("seeking: %ld\n", info->seek);
+
+                int new_position;
+                gulong milliseconds =
+                    mad_timer_count(info->duration, MAD_UNITS_MILLISECONDS);
+                if (info->seek >= milliseconds)
+                    info->seek = milliseconds;
+
+                mad_timer_set(&info->pos, 0, info->seek, 1000); // in millisecond
                 new_position =
-                    ((double) info->seek / (double) seconds) * info->size;
+                    ((double) info->seek / (double) milliseconds) * info->size;
 
                 if(new_position < 0)
                     new_position = 0;
-#ifdef DEBUG
-                g_message("seeking to: %d bytes", new_position);
-#endif
-                if (vfs_fseek(info->infile, new_position, SEEK_SET) == -1)
+
+                AUDDBG("seeking to: %d bytes\n", new_position);
+
+                if (aud_vfs_fseek(info->infile, new_position, SEEK_SET) == -1)
                     audmad_error("failed to seek to: %d", new_position);
                 mad_frame_mute(&frame);
                 mad_synth_mute(&synth);
@@ -593,23 +480,22 @@ gpointer decode_loop(gpointer arg)
                         continue;
                     }
                 }
-#ifdef DEBUG
-                g_message("(recovered) error decoding header %d: %s",
+
+                AUDDBG("(recovered) error decoding header %d: %s\n",
                           info->current_frame,
                           mad_stream_errorstr(&stream));
-#endif                          /* DEBUG */
+
                 continue;
             }
 
             info->bitrate = frame.header.bitrate;
 
-            if (!audmad_config.show_avg_vbr_bitrate && info->vbr && (iteration % 40 == 0)) {
-#ifdef DEBUG
+            if (!audmad_config->show_avg_vbr_bitrate && info->vbr && (iteration % 40 == 0)) {
+
 #ifdef DEBUG_INTENSIVELY
-                g_message("decode vbr tlen = %d", tlen);
+                AUDDBG("decode vbr tlen = %d\n", tlen);
 #endif
-#endif
-                mad_plugin->set_info(info->title,
+                info->playback->set_params(info->playback, info->title,
                                      (tlen == 0 || info->size <= 0) ? -1 : tlen,
                                      info->bitrate, info->freq, info->channels);
             }
@@ -618,11 +504,11 @@ gpointer decode_loop(gpointer arg)
             if (mad_frame_decode(&frame, &stream) == -1) {
                 if (!MAD_RECOVERABLE(stream.error))
                     break;
-#ifdef DEBUG
-                g_message("(recovered) error decoding frame %d: %s",
+
+                AUDDBG("(recovered) error decoding frame %d: %s\n",
                           info->current_frame,
                           mad_stream_errorstr(&stream));
-#endif                          /* DEBUG */
+
             }
 
             info->current_frame++;
@@ -630,22 +516,22 @@ gpointer decode_loop(gpointer arg)
             if (info->freq != frame.header.samplerate
                 || info->channels !=
                 (guint) MAD_NCHANNELS(&frame.header)) {
-#ifdef DEBUG
-                g_message("change in audio type detected");
-                g_message("old: frequency = %d, channels = %d", info->freq,
+
+                AUDDBG("change in audio type detected\n");
+                AUDDBG("old: frequency = %d, channels = %d\n", info->freq,
                           info->channels);
-                g_message("new: frequency = %d, channels = %d",
+                AUDDBG("new: frequency = %d, channels = %d\n",
                           frame.header.samplerate,
                           (guint) MAD_NCHANNELS(&frame.header));
-#endif                          /* DEBUG */
+
                 info->freq = frame.header.samplerate;
                 info->channels = MAD_NCHANNELS(&frame.header);
 
-                if(audmad_config.force_reopen_audio && check_audio_param(info)) {
+                if(audmad_config->force_reopen_audio && check_audio_param(info)) {
                     gint current_time = info->playback->output->output_time();
-#ifdef DEBUG
-                    g_message("re-opening audio due to change in audio type");
-#endif
+
+                    AUDDBG("re-opening audio due to change in audio type\n");
+
                     info->playback->output->close_audio();
                     if (!info->playback->output->open_audio(info->fmt, info->freq,
                                                             info->channels)) {
@@ -668,6 +554,7 @@ gpointer decode_loop(gpointer arg)
             mad_stream_sync(&stream);
 
             write_output(info, &synth.pcm, &frame.header);
+
             mad_timer_add(&info->pos, frame.header.duration);
         }
     }
@@ -684,17 +571,12 @@ gpointer decode_loop(gpointer arg)
         info->playback->output->buffer_free();
         info->playback->output->buffer_free();
         while (info->playback->output->buffer_playing()) {
-#ifdef DEBUG
-            g_message("f: buffer_playing=%d",
-                      info->playback->output->buffer_playing());
-#endif
-            g_get_current_time(&sleeptime);
-            sleeptime.tv_usec += 500000;
-            if(sleeptime.tv_usec >= 1000000) {
-                sleeptime.tv_sec += 1;
-                sleeptime.tv_usec -= 1000000;
-            }
 
+            AUDDBG("f: buffer_playing=%d\n", info->playback->output->buffer_playing());
+
+            g_get_current_time(&sleeptime);
+            g_time_val_add(&sleeptime, 500000);
+            
             g_mutex_lock(mad_mutex);
             g_cond_timed_wait(mad_cond, mad_mutex, &sleeptime);
             g_mutex_unlock(mad_mutex);
@@ -704,17 +586,15 @@ gpointer decode_loop(gpointer arg)
 
         }
     }
-#ifdef DEBUG
-    g_message("e: decode");
-#endif                          /* DEBUG */
 
-    bmp_title_input_free(info->tuple);
+    AUDDBG("e: decode\n");
+
+    aud_tuple_free(info->tuple);
     info->tuple = NULL;
 
     info->playback->output->close_audio();
     g_mutex_lock(mad_mutex);
     info->playback->playing = 0;
     g_mutex_unlock(mad_mutex);
-    g_thread_exit(0);
     return NULL; /* dummy */
 }
