@@ -55,7 +55,6 @@
 
 #include <fcntl.h>
 #include <errno.h>
-#include <audacious/util.h>
 
 #include "input.h"
 #include "replaygain.h"
@@ -138,14 +137,14 @@ mad_ucs4len(id3_ucs4_t *ucs)
 id3_ucs4_t *
 mad_ucs4dup(id3_ucs4_t *org)
 {
-    id3_ucs4_t *new = NULL;
+    id3_ucs4_t *res;
     size_t len = mad_ucs4len(org);
 
-    new = g_malloc0((len + 1) * sizeof(id3_ucs4_t));
-    memcpy(new, org, len * sizeof(id3_ucs4_t));
-    *(new + len) = 0; //terminate
+    res = g_malloc0((len + 1) * sizeof(id3_ucs4_t));
+    memcpy(res, org, len * sizeof(id3_ucs4_t));
+    *(res + len) = 0; //terminate
 
-    return new;
+    return res;
 }
 
 #define BYTES(x) ((x) * sizeof(id3_ucs4_t))
@@ -189,26 +188,26 @@ mad_parse_genre(const id3_ucs4_t *string)
             }
             else if (ptr <= tail && *ptr != 0) {
                 // reference to an id3v1 genre code
-                for(end = ptr; *end != ')' && *end != 0;) {
+                for(tmp_len = 0, end = ptr; *end != ')' && *end != 0;) {
                     end++;
+                    tmp_len++;
                 }
 
-                tmp = g_malloc0(BYTES(end - ptr + 1));
-                memcpy(tmp, ptr, BYTES(end - ptr));
-                *(tmp + (end - ptr)) = 0; //terminate
-                ptr += end - ptr;
+                tmp = g_malloc0(BYTES(tmp_len + 1));
+                memcpy(tmp, ptr, BYTES(tmp_len));
+                *(tmp + tmp_len) = 0; //terminate
+                ptr += tmp_len;
 
+                /* id3_genre_name may, in some cases, return the given string
+                 * so we must free it after we're done, not before.
+                 */
                 genre = (id3_ucs4_t *)id3_genre_name((const id3_ucs4_t *)tmp);
 
-                g_free(tmp);
-                tmp = NULL;
-
                 tmp_len = mad_ucs4len(genre);
-
                 memcpy(ret + ret_len, genre, BYTES(tmp_len));
-
                 ret_len += tmp_len;
                 *(ret + ret_len) = 0; //terminate
+                g_free(tmp);
             }
         }
         else {
@@ -228,34 +227,34 @@ mad_parse_genre(const id3_ucs4_t *string)
 
             if(is_num) {
                 AUDDBG("is_num!\n");
+                tmp_len = end - ptr;
+                tmp = g_malloc0(BYTES(tmp_len + 1));
+                memcpy(tmp, ptr, BYTES(tmp_len));
+                *(tmp + tmp_len) = 0; //terminate
+                ptr += tmp_len;
 
-                tmp = g_malloc0(BYTES(end - ptr + 1));
-                memcpy(tmp, ptr, BYTES(end - ptr));
-                *(tmp + (end - ptr)) = 0; //terminate
-                ptr += end - ptr;
-
+                /* id3_genre_name may, in some cases, return the given string
+                 * so we must free it after we're done, not before.
+                 */
                 genre = (id3_ucs4_t *)id3_genre_name((const id3_ucs4_t *)tmp);
                 AUDDBG("genre length = %d\n", mad_ucs4len(genre));
-
-                g_free(tmp);
-                tmp = NULL;
-
+                
                 tmp_len = mad_ucs4len(genre);
-
                 memcpy(ret + ret_len, genre, BYTES(tmp_len));
-
                 ret_len += tmp_len;
                 *(ret + ret_len) = 0; //terminate
+                g_free(tmp);
             }
             else { // plain text
+                tmp_len = end - ptr;
                 AUDDBG("plain!\n");
                 AUDDBG("ret_len = %d\n", ret_len);
-                AUDDBG("end - ptr = %d\n", BYTES(end - ptr));
+                AUDDBG("end - ptr = %d\n", BYTES(tmp_len));
 
-                memcpy(ret + ret_len, ptr, BYTES(end - ptr));
-                ret_len = ret_len + (end - ptr);
+                memcpy(ret + ret_len, ptr, BYTES(tmp_len));
+                ret_len = ret_len + tmp_len;
                 *(ret + ret_len) = 0; //terminate
-                ptr += (end - ptr);
+                ptr += tmp_len;
             }
         }
     }
@@ -352,7 +351,7 @@ input_alloc_tag(struct mad_info_t *info)
 static void
 input_read_tag(struct mad_info_t *info)
 {
-    gchar *string = NULL;
+    gchar *string;
     Tuple *tuple;
     glong curpos = 0;
 
@@ -360,7 +359,7 @@ input_read_tag(struct mad_info_t *info)
 
     if (info->tuple != NULL)
         aud_tuple_free(info->tuple);
-        
+    
     tuple = aud_tuple_new_from_filename(info->filename);
     info->tuple = tuple;
 
@@ -393,11 +392,9 @@ input_read_tag(struct mad_info_t *info)
     if (string) {
         aud_tuple_associate_int(tuple, FIELD_TRACK_NUMBER, NULL, atoi(string));
         g_free(string);
-        string = NULL;
     }
 
     // year
-    string = NULL;
     string = input_id3_get_string(info->tag, ID3_FRAME_YEAR);   //TDRC
     if (!string)
         string = input_id3_get_string(info->tag, "TYER");
@@ -405,7 +402,6 @@ input_read_tag(struct mad_info_t *info)
     if (string) {
         aud_tuple_associate_int(tuple, FIELD_YEAR, NULL, atoi(string));
         g_free(string);
-        string = NULL;
     }
 
     // length
@@ -414,7 +410,6 @@ input_read_tag(struct mad_info_t *info)
         aud_tuple_associate_int(tuple, FIELD_LENGTH, NULL, atoi(string));
         AUDDBG("input_read_tag: TLEN = %d\n", atoi(string));
         g_free(string);
-        string = NULL;
     } else
         aud_tuple_associate_int(tuple, FIELD_LENGTH, NULL, -1);
     
@@ -439,7 +434,7 @@ input_process_remote_metadata(struct mad_info_t *info)
     gboolean metadata = FALSE;
 
     if(info->remote && mad_timer_count(info->duration, MAD_UNITS_SECONDS) <= 0){
-        gchar *tmp = NULL;
+        gchar *tmp;
 
 #ifdef DEBUG_INTENSIVELY
         AUDDBG("process_remote_meta\n");
@@ -452,28 +447,20 @@ input_process_remote_metadata(struct mad_info_t *info)
         tmp = aud_vfs_get_metadata(info->infile, "track-name");
         if(tmp){
             metadata = TRUE;
-            gchar *scratch;
-
-            scratch = aud_str_to_utf8(tmp);
+            gchar *scratch = aud_str_to_utf8(tmp);
             aud_tuple_associate_string(info->tuple, FIELD_TITLE, NULL, scratch);
             g_free(scratch);
-
             g_free(tmp);
-            tmp = NULL;
         }
 
         tmp = aud_vfs_get_metadata(info->infile, "stream-name");
         if(tmp){
             metadata = TRUE;
-            gchar *scratch;
-
-            scratch = aud_str_to_utf8(tmp);
+            gchar *scratch = aud_str_to_utf8(tmp);
             aud_tuple_associate_string(info->tuple, FIELD_ALBUM, NULL, scratch);
             aud_tuple_associate_string(info->tuple, -1, "stream", scratch);
             g_free(scratch);
-
             g_free(tmp);
-            tmp = NULL;
         }
 
         if (metadata)
@@ -482,8 +469,8 @@ input_process_remote_metadata(struct mad_info_t *info)
             gchar *realfn = g_filename_from_uri(info->filename, NULL, NULL);
             gchar *tmp2 = g_path_get_basename(realfn ? realfn : info->filename); // info->filename is uri. --yaz
             tmp = aud_str_to_utf8(tmp2);
-            g_free(tmp2); tmp2 = NULL;
-            g_free(realfn); realfn = NULL;
+            g_free(tmp2);
+            g_free(realfn);
 //            tmp = g_strdup(g_basename(info->filename)); //XXX maybe ok. --yaz
         }
 
@@ -521,7 +508,7 @@ input_get_info(struct mad_info_t *info, gboolean fast_scan)
     input_read_tag(info);
 
     if(!info->remote) { // reduce startup delay
-        read_replaygain(info);
+        audmad_read_replaygain(info);
     }
 
     /* scan mp3 file, decoding headers */
@@ -587,37 +574,27 @@ input_term(struct mad_info_t * info)
 {
     AUDDBG("f: input_term\n");
 
-    if (info->title)
-        g_free(info->title);
-    if (info->url)
-        g_free(info->url);
-    if (info->filename)
-        g_free(info->filename);
+    g_free(info->title);
+    g_free(info->url);
+    g_free(info->filename);
     if (info->infile)
         aud_vfs_fclose(info->infile);
     if (info->id3file)
         id3_file_close(info->id3file);
 
-    if (info->replaygain_album_str)
-        g_free(info->replaygain_album_str);
-    if (info->replaygain_track_str)
-        g_free(info->replaygain_track_str);
-    if (info->replaygain_album_peak_str)
-        g_free(info->replaygain_album_peak_str);
-    if (info->replaygain_track_peak_str)
-        g_free(info->replaygain_track_peak_str);
-    if (info->mp3gain_undo_str)
-        g_free(info->mp3gain_undo_str);
-    if (info->mp3gain_minmax_str)
-        g_free(info->mp3gain_minmax_str);
+    g_free(info->replaygain_album_str);
+    g_free(info->replaygain_track_str);
+    g_free(info->replaygain_album_peak_str);
+    g_free(info->replaygain_track_peak_str);
+    g_free(info->mp3gain_undo_str);
+    g_free(info->mp3gain_minmax_str);
 
     if (info->tuple) {
         aud_tuple_free(info->tuple);
         info->tuple = NULL;
     }
 
-    if (info->prev_title)
-        g_free(info->prev_title);
+    g_free(info->prev_title);
 
     /* set everything to zero in case it gets used again. */
     memset(info, 0, sizeof(struct mad_info_t));
