@@ -1,11 +1,28 @@
+/*
+ * paranormal: iterated pipeline-driven visualization plugin
+ * Copyright (c) 2006, 2007 William Pitcock <nenolod@dereferenced.org>
+ * Portions copyright (c) 2001 Jamie Gennis <jgennis@mindspring.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; under version 2 of the License.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
+
 /* FIXME: allow for only using an xform on part of the img? */
 /* FIXME: perhaps combine these into a single vector field
    so that only 1 apply_xform needs to be done for as many
    of these as someone wants to use */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <math.h>
 
@@ -524,12 +541,9 @@ inline void
 xform_trans_literal (struct xform_vector *vfield, gint x, gint y,
 	expression_t *expr, symbol_dict_t *dict)
 {
-  gdouble rf, df;
-  gdouble *xf, *yf;
   gint xn, yn;
-
-  xf = dict_variable(dict, "x");
-  yf = dict_variable(dict, "y");
+  gdouble *xf = dict_variable(dict, "x");
+  gdouble *yf = dict_variable(dict, "y");
 
   /* Points (xf, yf) must be in a (-1..1) square. */
   *xf = 2.0 * x / (pn_image_data->width - 1) - 1.0;
@@ -563,8 +577,6 @@ xform_movement_exec (const struct pn_actuator_option *opts,
     {
       gint i, j;
       gdouble *rf, *df;
-      gdouble xf, yf;
-      gint xn, yn;
       expression_t *expr;
       symbol_dict_t *dict;
 
@@ -676,8 +688,6 @@ xform_dynmovement_exec (const struct pn_actuator_option *opts,
   PnDynMovementData *d = (PnDynMovementData *) odata;
   gint i, j;
   gdouble *rf, *df;
-  gdouble xf, yf;
-  gint xn, yn;
   void (*transform_func)(struct xform_vector *, gint, gint, expression_t *, symbol_dict_t *) = 
         opts[4].val.bval == TRUE ? xform_trans_polar : xform_trans_literal;
   gboolean make_table = FALSE;
