@@ -28,8 +28,6 @@
 
 #include <gtk/gtk.h>
 #include <audacious/plugin.h>
-#include <audacious/playlist.h>
-#include <audacious/util.h>
 #include <audacious/i18n.h>
 
 #include <pulse/pulseaudio.h>
