@@ -21,11 +21,8 @@
 
 */
 
-#if HAVE_CONFIG_H
-#  include <config.h>
-#endif
+#include <config.h>
 
-#include "audacious/vfs.h"
 #include <stdlib.h>
 #include <string.h>
 
@@ -777,9 +774,9 @@ size_t mid_song_read_wave(MidSong *song, void *ptr, size_t size)
       song->current_event++;
     }
     if (song->current_event->time > end_sample)
-      compute_data(song, (sint8 **)&ptr, end_sample-song->current_sample);
+      compute_data(song, (sint8 **)(void*)&ptr, end_sample-song->current_sample);
     else
-      compute_data(song, (sint8 **)&ptr, song->current_event->time-song->current_sample);
+      compute_data(song, (sint8 **)(void*)&ptr, song->current_event->time-song->current_sample);
   }
   return samples * song->bytes_per_sample;
 }
