@@ -42,10 +42,7 @@
 #include <string.h>
 
 #include <audacious/plugin.h>
-#include <audacious/util.h>
-#include <audacious/main.h>
 #include <audacious/output.h>
-#include <audacious/strings.h>
 #include <audacious/i18n.h>
 #include <audacious/id3tag.h>
 
