@@ -1,7 +1,7 @@
 /*
  *    Plugin configure dialog.
  */
-#include "audacious/plugin.h"
+#include <audacious/plugin.h>
 
 void vtx_config(void)
 {
