@@ -1,13 +1,7 @@
-#include <audacious/plugin.h>
-#include <audacious/util.h>
-#include <audacious/output.h>
-#include <audacious/titlestring.h>
-#include <audacious/vfs.h>
-#include <audacious/strings.h>
-#include <audacious/i18n.h>
-
 #include "ayemu.h"
 #include "vtx.h"
+#include <audacious/output.h>
+#include <audacious/i18n.h>
 
 void vtx_file_info(gchar *filename)
 {
@@ -39,7 +33,7 @@ void vtx_file_info(gchar *filename)
 	      "Player Freq:%P\n"
 	      "Year: %y");
       
-      box = xmms_show_message (head,
+      box = audacious_info_dialog (head,
 			       body,
 			       _("Ok"), FALSE, NULL, NULL);
       
