@@ -37,10 +37,7 @@
 
 #include <audacious/plugin.h>
 #include <audacious/output.h>
-#include <audacious/util.h>
-#include <audacious/strings.h>
 #include <audacious/i18n.h>
-#include <audacious/main.h>
 
 #include "avcodec.h"
 #include "avformat.h"
